public class Filter
{
	double[] y_effects;
	double[] x_effects;

	public Filter(Polynomial poles, Polynomial zeroes)
	{
		y_effects = translatePolynomial(poles);
		x_effects = translatePolynomial(zeroes);
	}

	double[] translatePolynomial(Polynomial p)
	{
		// find max delay.  Assume causal.
		int delay = 0;
		for (int i=0;i<p.getSize();i++)
		{
			SimplePolyTerm spt = p.getTerm(i);
			int power = spt.getPower();
			if (power < delay)
				delay = power;
		}
		double[] result = new double[(-1 * delay) + 1];
		for (int i=0;i<p.getSize();i++)
		{
			SimplePolyTerm spt = p.getTerm(i);
			int power = spt.getPower();
			double coeff = spt.getCoefficient();
			result[-1 * power] = coeff;
		}
		return result;		
	}

	public double[] run(double[] data)
	{
		double[] result = new double[data.length];
		for (int i=0;i<data.length;i++)
		{
			double temp = 0.0;
			// first apply zeroes (the FIR portion)
			for (int j=0;j<x_effects.length;j++)
			{
				if ((i-j) >= 0)
					temp = temp + (x_effects[j] * data[i-j]);
			}
			// now apply poles (the IIR portion).  Note that we are 
			// going to subtract, and we start from 1.
			for (int j=1;j<y_effects.length;j++)
			{
				if ((i-j) >= 0)
					temp = temp - (y_effects[j] * result[i-j]);
			}
			result[i] = temp;
		}
		return result;
	}
}