import java.awt.*;
import java.awt.event.*;

public class Graph extends Container
                           implements MouseListener, MouseMotionListener, ActionListener
{
	double[] data = null;
	boolean continuous = true;
	double start = 0;
	double step = 1;
	double max = 0.0;
	double maxused = 0.0;
	Color datacolor;

	int GRAPH_HEIGHT;
	int GRAPH_WIDTH;
	int REAL_HEIGHT;
	int REAL_WIDTH;

	boolean originbottom = false;

	static final int BUFFER = 5;
	static final int ARROW_DELTA = 4;

	Label xlabel;
	Label ylabel;

	public Graph(int height, int width)
	{
		REAL_HEIGHT = height + 40;
		REAL_WIDTH = width + 100;
		setSize(REAL_HEIGHT, REAL_WIDTH);
		GRAPH_HEIGHT = height-1;
		GRAPH_WIDTH = width-1;		
		data = new double[100];
		int temp = -1;
		for (int i=0;i<data.length;i++)
		{
			data[i] = temp;
			temp = -1 * temp;
		}

		xlabel = new Label("X");
		xlabel.setBounds(5,  REAL_HEIGHT-35, REAL_WIDTH/2 - 5, 30);
		add(xlabel);
		xlabel.setVisible(true);

		ylabel = new Label("Y");
		ylabel.setBounds(REAL_WIDTH/2,  REAL_HEIGHT-35, REAL_WIDTH/2 - 5, 30);
		add(ylabel);
		ylabel.setVisible(true);

		datacolor = Color.green;

		addMouseListener(this);
		addMouseMotionListener(this);
	}

	public void setTitle(String s)
	{
		getGraphics().drawString(s, 5, 20);
	}

	public int getHeight()
	{
		return REAL_HEIGHT;
	}

	public int getWidth()
	{
		return REAL_WIDTH;
	}

	public void originIsAtBottom(boolean b)
	{
		originbottom = b;
	}

	public void setData(double[] d, int start, int length)
	{
		data = new double[length];
		System.arraycopy(d, start, data, 0, length);
	}

	public void setContinuous(boolean c)
	{
		continuous = c;
	}

	public void setDataRegion(double start, double step)
	{
		this.start = start;
		this.step = step;
	}

	public void setDynamicRange(double m)
	{
		max = m;
	}

	public void setDataColor(Color c)
	{
		datacolor = c;
	}

	

	public void paint(Graphics g)
	{
		// first draw a border
		g.drawLine(0, 0, GRAPH_WIDTH, 0);
		g.drawLine(GRAPH_WIDTH, 0, GRAPH_WIDTH, GRAPH_HEIGHT);
		g.drawLine(GRAPH_WIDTH, GRAPH_HEIGHT, 0, GRAPH_HEIGHT);
		g.drawLine(0, GRAPH_HEIGHT, 0, 0);

		// first draw the X axis
		int y_axis = GRAPH_HEIGHT/2;
		if (originbottom)
			y_axis = GRAPH_HEIGHT;
		g.drawLine(BUFFER, y_axis, GRAPH_WIDTH - BUFFER, y_axis);
		// right arrow
		g.drawLine(GRAPH_WIDTH - BUFFER - ARROW_DELTA, y_axis - ARROW_DELTA, GRAPH_WIDTH - BUFFER, y_axis); 
		g.drawLine(GRAPH_WIDTH - BUFFER - ARROW_DELTA, y_axis + ARROW_DELTA, GRAPH_WIDTH - BUFFER, y_axis); 
		// left arrow
		g.drawLine(BUFFER + ARROW_DELTA, y_axis - ARROW_DELTA, BUFFER, y_axis); 
		g.drawLine(BUFFER + ARROW_DELTA, y_axis + ARROW_DELTA, BUFFER, y_axis); 

		double pixels_per_datum = (double)GRAPH_WIDTH / (double)data.length;
		double tempmax = max;
		if (tempmax==0.0)
		{
			// figure out what the max is
			for (int i=0;i<data.length;i++)
			{
				if (Math.abs(data[i]) > tempmax)
					tempmax = Math.abs(data[i]);
			}
		}
		maxused = tempmax;
		double pixels_per_unitvalue = (double)GRAPH_HEIGHT / tempmax;
		if (!originbottom)
		{
			pixels_per_unitvalue = (double)(GRAPH_HEIGHT/2) / tempmax;
		}

		double dynamicrange = tempmax;
		if (!originbottom)
			dynamicrange = tempmax * 2;

		double step = 1.0;
		// figure out what my step should be from dynamicrange
		if (dynamicrange < 0.5)
			step = 0.1;
		else if (dynamicrange < 2.0)
			step = 0.25;
		else if (dynamicrange < 4.0)
			step = 0.50;
		else 
		{
			long dr = (long)(dynamicrange / 10.0);
			step = dr + 1;
			if (step < 0)
				step = 0;
		}

		double count = 0.0;
		int marker = y_axis;
		// skip the number if there was overflow in the step
		if (step==0)
			marker = 0;
		double value = 0.0;
		while ((marker > 0) && (pixels_per_unitvalue > 0))
		{
			count += 1.0;
			g.drawLine(GRAPH_WIDTH, marker, GRAPH_WIDTH+5, marker);
			g.drawString(" "+value, GRAPH_WIDTH+7, marker+5);
			value += step;
			marker = y_axis - (int)(pixels_per_unitvalue * step * count);
//System.out.println("Marker: "+marker+" Step: "+step+" Dynamic Range: "+dynamicrange);
//try{Thread.sleep(100);} catch(Exception e){}
		}


		g.setColor(datacolor);
		int lastx = 0;
		int lasty = GRAPH_HEIGHT/2;
		for (int i=0;i<data.length;i++)
		{
			int x = (int)((double)i * pixels_per_datum);
			int y = y_axis - (int)(data[i] * pixels_per_unitvalue);
			if (continuous)
			{
				g.drawLine(lastx, lasty, x, y);
				lastx = x;
				lasty = y;
			}
			else
			{
				g.drawLine(x, GRAPH_HEIGHT/2, x, y);
			}
		}
		

		
		
	}



	public void actionPerformed(ActionEvent ae)
	{
		Object o = ae.getSource();
	}





  public void mouseClicked(MouseEvent me)
  {
  }

  public void mouseEntered(MouseEvent me)
  {
    //System.out.println("ENTERED");
  }

  public void mouseExited(MouseEvent me)
  {
    //System.out.println("EXITED");
  }

  public void mousePressed(MouseEvent me)
  {
    //System.out.println("PRESSED");
  }

  public void mouseReleased(MouseEvent me)
  {
    //System.out.println("RELEASED");
  }

  public void mouseDragged(MouseEvent me)
  {
  }
	
  public void mouseMoved(MouseEvent me)
  {
	xlabel.setText("X = "+(start + step*me.getX()));
	ylabel.setText("Y ??? (MAX "+maxused+")");
  }

}