;*---------------------------------------------------------------------------
;* Copyright (C) 2005 Dallas Semiconductor Corporation, All Rights Reserved.
;*
;* Permission is hereby granted, free of charge, to any person obtaining a
;* copy of this software and associated documentation files (the "Software"),
;* to deal in the Software without restriction, including without limitation
;* the rights to use, copy, modify, merge, publish, distribute, sublicense,
;* and/or sell copies of the Software, and to permit persons to whom the
;* Software is furnished to do so, subject to the following conditions:
;*
;* The above copyright notice and this permission notice shall be included
;* in all copies or substantial portions of the Software.
;*
;* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
;* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
;* MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
;* IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
;* OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
;* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
;* OTHER DEALINGS IN THE SOFTWARE.
;*
;* Except as contained in this notice, the name of Dallas Semiconductor
;* shall not be used except as stated in the Dallas Semiconductor
;* Branding Policy.
;*---------------------------------------------------------------------------
#include "iomaxq200x.h"

#define MCNT M1[0]
#define MA   M1[1]
#define MB   M1[2]
#define MC2  M1[3]
#define MC1  M1[4]
#define MC0  M1[5]

 module fred

 rseg DATA_N:DATA
sampletable: DS8 512
sampleindex: DS8 1

 rseg CODE:CODE
 public filtersample

;******************************************************************************
;
; Runs the selected filter over the audio data and outputs the current sample.
;
; Inputs: A[0] current sample
;
; Outputs: A[0] resulting output sample
;
; Destroyed: Anything that IAR says I can.  I preserve DP[1] and DPC.
;
;******************************************************************************
filtersample:
    push  DP[1]                        ; preserve IAR's software stack
    push  DPC                          ; IAR probably needs this preserved too
    move  AP, #0                       ; select accumulator 0
;   xor   #8000h                       ; set '0' for our signal
 sub #2048
    move  DPC, #10h                    ; DP[0] byte mode, BP word mode
    move  BP, #W:sampletable           ; get the start of the sample table
    move  DP[0], #B:sampleindex        ; point to current index into the sample 
    move  AP, #1                       ; select accumulator 1
    move  ACC, @DP[0]                  ; get current table index
    move  Offs, ACC                    ; put it in the offset register
    add   #1                           ; increment the current index
    move  @DP[0], ACC                  ; restore the table pointer
    move  @BP[Offs], A[0]              ; store the current sample
    
    move  MCNT, #22h                   ; signed, mult-accum, clear regs first
    
filterloop:
    ;
    ; Unroll the filter loop for speed.
    ;
    
    
; this one is pretty good    
; dc16 14, 15, 0x1333, 0xccc, 0x53a, 0x922, 0x4c7, 0x364, 0x2b8, 0x211, 0x12b, 0x100
; dc16 0x109, 0xc6, 0x76, 0xffdf, 0xd5
    
; better...though still a bit 'wobbly' (low freq noise)
; dc16 14, 25, 0x0, 0x6, 0x24, 0x8d, 0x18e, 0x365, 0x5f2, 0x8960xa62, 0xaa5, 0x954, 0x705, 0x48c, 0x28a, 0x138, 0x810x2d, 0xd, 0x3

; better still...
;  dc16 14, 21, 0x2, 0x14, 0x4e, 0xd7, 0x1d6, 0x357, 0x537, 0x7220x8a5, 0x95c, 0x915, 0x7ea, 0x630, 0x451, 0x2ab, 0x1720xac, 0x42, 0x14, 0x4, 0x0

; dc16 14, 27, 0x16, 0x48, 0xad, 0x140, 0x200, 0x2d3, 0x3b1, 0x4790x526, 0x5a1, 0x5ea, 0x5f1, 0x5c0, 0x553, 0x4bf, 0x40c0x352, 0x299, 0x1f4, 0x163, 0xf0, 0x97, 0x58, 0x2e0x15, 0x7, 0x2

    move  MA, #0x16
    move  MB, @BP[Offs--]
    move  MA, #0x48
    move  MB, @BP[Offs--]
    move  MA, #0xad
    move  MB, @BP[Offs--]
    move  MA, #0x140
    move  MB, @BP[Offs--]
    move  MA, #0x200
    move  MB, @BP[Offs--]
    move  MA, #0x2d3
    move  MB, @BP[Offs--]
    move  MA, #0x3b1
    move  MB, @BP[Offs--]
    move  MA, #0x479
    move  MB, @BP[Offs--]
    move  MA, #0x526
    move  MB, @BP[Offs--]
    move  MA, #0x5a1
    move  MB, @BP[Offs--]
    move  MA, #0x5ea
    move  MB, @BP[Offs--]
    move  MA, #0x5f1
    move  MB, @BP[Offs--]
    move  MA, #0x5c0
    move  MB, @BP[Offs--]
    move  MA, #0x553
    move  MB, @BP[Offs--]
    move  MA, #0x4bf
    move  MB, @BP[Offs--]
    move  MA, #0x40c
    move  MB, @BP[Offs--]
    move  MA, #0x352
    move  MB, @BP[Offs--]
    move  MA, #0x299
    move  MB, @BP[Offs--]
    move  MA, #0x1f4
    move  MB, @BP[Offs--]
    move  MA, #0x163
    move  MB, @BP[Offs--]
    move  MA, #0xf0
    move  MB, @BP[Offs--]
    move  MA, #0x97
    move  MB, @BP[Offs--]
    move  MA, #0x58
    move  MB, @BP[Offs--]
    move  MA, #0x2e
    move  MB, @BP[Offs--]
    move  MA, #0x15
    move  MB, @BP[Offs--]
    move  MA, #0x7
    move  MB, @BP[Offs--]
    move  MA, #0x2
    move  MB, @BP[Offs--]

    nop

    move  A[2], MC2                    ; get MAC result HIGH
    move  A[1], MC1                    ; get MAC result MID
    move  A[0], MC0                    ; get MAC result LOW

    move  APC, #0C2h                   ; clear AP, roll modulo 4, auto-dec AP
    move  LC[1], #14
shift_loop:    
    ; 
    ; Because we use fixed point precision, we need to shift to get a real 
    ; sample value.  This is not as efficient as it could be.  If we had a 
    ; dedicated filter, we might make use of the shift-by-2 and shift-by-4
    ; instructions available on MAXQ.
    ; 
    move  AP, #2                       ; select HIGH MAC result
    move  c, #0                        ; clear carry
    rrc                                ; shift HIGH MAC result
    rrc                                ; shift MID MAC result   
    rrc                                ; shift LOW MAC result   
    djnz  LC[1], shift_loop            ; shift to get result in A[0]
    move APC, #0                       ; restore accumulator normalcy
    move AP, #0                        ; use accumulator 0
;   xor  #8000h                        ; level shift output result
 add #2048
    pop  DPC                           ; restore data pointer control
    pop  DP[1]                         ; restore IAR stack pointer
    ret
 
