/*******************************************************************************
 *
 *  Copyright (C) 2005 Dallas/Maxim Semiconductor Corporation. 
 *  All rights Reserved. Printed in U.S.A.
 *
 *  This software is protected by copyright laws of the United States and of 
 *  foreign countries.  This material may also be protected by patent laws of
 *  the United States and of foreign countries.  This software is furnished 
 *  under a license agreement and/or a nondisclosure agreement and may only 
 *  be used or copied in accordance with the terms of those agreements.  The 
 *  mere transfer of this software does not imply any licenses of trade 
 *  secrets, proprietary technology, copyrights, patents, trademarks, 
 *  maskwork rights, or any other form of intellectual property whatsoever. 
 *  Dallas/Maxim Semiconductor retains all ownership rights.
 *
 *     Description: MAXQ7654 (UC21) EV Kit DAC Simple Test Program
 *
 *           Tests: MAXQ7654 DACs by producing a sine wave at both DAC outputs
 *																	
 *        Filename: main.c
 *
 *          Author: KLA (with code from DV)
 *
 *        Compiler: IAR
 *
 *         Version: 1.12B
 *
 *         Created: 4/4/05  
 *							
 *							
 *******************************************************************************/
   
#include <stdio.h>
#include <string.h>
#include <iomaxq.h>
#include <iomaxq20_uc21_REV.h>              // Header file for UC21
#include <math.h>

__intrinsic void __disable_interrupt(void);
__intrinsic void __enable_interrupt(void);


/*
 * Assembler function that gets the audio sample at the specified offset
 * and runs it through the specified filter.  See the file 'data.asm' for
 * information on how the filters are stored.
 */
extern unsigned int filtersample(unsigned int sample);


// at 8MHz, this reload gives us about 200 us per step.  At 64 steps 
// in a sine wave, that a period around 12.5 ms.
#define TIMERRELOAD 1563
    
#define SIN_WAVE_STEPS 64
float static_sin_data[SIN_WAVE_STEPS];
    
#define   REF_BG            0x0000        // DAC Section Defines
#define   REF_AVDD          0x0001             
#define   DAC1_OPAMP        0x0000
#define   DAC1_COMP         0x0004   
#define   DAC2_OPAMP        0x0000
#define   DAC2_COMP         0x0008          
#define   LOAD_PIN_RISE     0x0000
#define   LOAD_DACI_1       0x0010
#define   LOAD_DACI_2       0x0020
#define   LOAD_DACI_1_2     0x0030
#define   LOAD_PIN_FALL     0x0040
#define   LOAD_SQ_WAVE      0x0050
#define   ADC_OPAMP         0x0000
#define   ADC_COMP          0x0080

#define   AIN0          0x0000
#define   START_CONV    0x0008
#define   CONTINUOUS    0x0006
#define   VDD_REF       0x0020


/********************************************************************
 *
 * Function:    DAC #1 Enable      
 *
 * Description: Set the Analog Power Enable Bits for DAC #1  	
 *                                     
 * Input: 	    None     		       
 *						
 * Output: 	    None     	                                    
 *                     
 * Destroys:    APE DAC #1 Bits	    	   
 *                                 
 ********************************************************************/
void  DAC_1_Enable()
{           
  APE = APE | 0x0063;       // Enable DAC1, Internal Bandgap Reference and the DAC Reference Buffer           
}



/********************************************************************
 *
 * Function:    DAC #1 Disable      
 *
 * Description: Clear the Analog Power Enable Bits for DAC #1  	
 *                                     
 * Input: 	    None     		       
 *						
 * Output: 	    None     	                                    
 *                     
 * Destroys:    APE DAC #1 Bits	    	   
 *                                 
 ********************************************************************/
void  DAC_1_Disable()
{           
  APE = APE & ~0x0002;       // Disable DAC1           
}

/********************************************************************
 *
 * Function:    DAC #2 Enable      
 *
 * Description: Set the Analog Power Enable Bits for DAC #2  	
 *                                     
 * Input: 	    None     		       
 *						
 * Output: 	    None     	                                    
 *                     
 * Destroys:    APE DAC #2 Bits	    	   
 *                                 
 ********************************************************************/
void  DAC_2_Enable()
{           
  APE = APE | 0x0065;       // Enable DAC2, Internal Bandgap Reference and the DAC Reference Buffer           
}

void  ADC_Enable()
{                                       
  APE_bit.ADCE  = 1;                    // Enable the ADC  
  APE_bit.OAE   = 1;                    // Enable the ADC input Op Amp         
  APE_bit.BGRE  = 1;                    // Enable the internal Bandgap Reference
  APE_bit.ARBE  = 1;                    // Enable the ADC Reference Buffer
}

unsigned int ADC_Convert_Poll (unsigned int Control_Reg)
{           
  ACNT = Control_Reg;             // Set the ADC parameters
  while( ACNT_bit.ADCBY == 1);    // Wait till ADC is not busy
  return ADCD;                    // Return the ADC result
}

/********************************************************************
 *
 * Function:    DAC #2 Disable      
 *
 * Description: Clear the Analog Power Enable Bits for DAC #2  	
 *                                     
 * Input: 	    None     		       
 *						
 * Output: 	    None     	                                    
 *                     
 * Destroys:    APE DAC #2 Bits	    	   
 *                                 
 ********************************************************************/
void  DAC_2_Disable()
{           
  APE = APE & ~0x0004;       // Disable DAC2           
}


/******************************************************************************
 * The interrupt handler for module 3, which is my timer 2 interrupt.
 */
int sinindex = 0; 
int sinnoise = 0xae42;
 
#pragma vector = 2
__interrupt void timer0Interrupt()
{
    int sample;
    int thisnoise;
    unsigned int inputsample;
    if (T2CNB0 & 0x0A)   // then it is timer 2 interrupt condition
    {
        // clear interrupt condition
        T2CNB0 = T2CNB0 & 0xF5;
        
        
        sample =  static_sin_data[sinindex++];
        sinnoise = ((sinnoise ^ 0x5C) * 31) + 0xabcd;
        thisnoise = sinnoise;
        if (thisnoise & 0x01)
        {
            thisnoise = thisnoise & 0x1ff;
        }
        else
        {
            thisnoise = -1 * (thisnoise & 0x1ff);
        }
        sample += thisnoise;
        if (sample < 0)
            sample = sample * -1;
        if (sample > 4095)
            sample = 8192 - sample;
        
        DACI1 = sample;        // Send value to DAC #1
        if (sinindex >= SIN_WAVE_STEPS)
            sinindex = 0;
            
            
        inputsample = ADC_Convert_Poll(AIN0 | START_CONV | CONTINUOUS); //VDD_REF
        DACI2 = filtersample(inputsample);
//        DACI2 = inputsample;
            
    }
}


/********************************************************************
 *
 * Function:     Main Program
 *
 * Description:  Calls the functions used in the program
 *                                     
 * Input: 	     None       
 *
 * Output: 	     None                                     
 *                     
 * Destroys:    DCNT, DACI1, DACI2         
 *                                 
 ********************************************************************/
 
 
void main()
{
  float  Offset, Amplitude, Steps;
  int i;
 
  CKCN = 0x80;                      // Enable the External 8 MHz Crystal Oscillator
  OSCC = 0x0F;
  
  ADC_Enable();
  DAC_1_Enable();                         // Enable DAC #1      
  DAC_2_Enable();                         // Enable DAC #2      

  DCNT = (REF_BG | DAC1_OPAMP | DAC2_OPAMP | LOAD_DACI_1_2 | ADC_OPAMP);

  Offset = 2048;                          // Set the sine wave offset from 0 (Range 0 to 2048)
  Amplitude = 1800;                       // Set the sine wave amplitude (Range 0 to 2047)
  Steps = SIN_WAVE_STEPS;                 // Enter the number of steps in the sine wave
  
  for (i=0;i<SIN_WAVE_STEPS;i++)
  {
      static_sin_data[i] = Offset + Amplitude * sin((float)i *__PI* 2/Steps);
  }
  
  PD1 = 0xFF;
  PO1 = 0;

  //
  // Set up timer number 2 to give us periodic interrupts, when we will 
  // output a new sinewave value.
  //
  T2V0 = 0xFFFF - TIMERRELOAD;
  T2R0 = 0xFFFF - TIMERRELOAD;
  T2C0 = 0x0002;    // no compare, don't want that interrupt firing
  
  T2CFG0 = 0x00;    // use clock divided by 128
  T2CNB0 = 0x00;
  T2CNA0 = 0x88;    // start the timer, and enable interrupts
  
  IMR |= 0x04;      // turn on interrupts for module 2
  __enable_interrupt();

  while(1)
  {        
    for (i = 0 ; i < SIN_WAVE_STEPS ; i++)
    {
      // output an 'alive' signal
      PO1 ^= 1;
    }
  }
}   // End of Program

